@@ -5,7 +5,6 @@ import string
 from collections import namedtuple
 
 import kubernetes
-import six
 from dagster import Array, Field, Noneable, StringSource
 from dagster import __version__ as dagster_version
 from dagster import check
@@ -120,7 +119,9 @@ def get_k8s_resource_requirements(tags):
 
     if not result.success:
         raise DagsterInvalidConfigError(
-            "Error in tags for {}".format(K8S_RESOURCE_REQUIREMENTS_KEY), result.errors, result,
+            "Error in tags for {}".format(K8S_RESOURCE_REQUIREMENTS_KEY),
+            result.errors,
+            result,
         )
 
     return result.value
@@ -140,7 +141,9 @@ def get_user_defined_k8s_config(tags):
 
         if not result.success:
             raise DagsterInvalidConfigError(
-                "Error in tags for {}".format(USER_DEFINED_K8S_CONFIG_KEY), result.errors, result,
+                "Error in tags for {}".format(USER_DEFINED_K8S_CONFIG_KEY),
+                result.errors,
+                result,
             )
 
         user_defined_k8s_config = result.value
@@ -172,7 +175,7 @@ class DagsterK8sJobConfig(
     namedtuple(
         "_K8sJobTaskConfig",
         "job_image dagster_home image_pull_policy image_pull_secrets service_account_name "
-        "instance_config_map postgres_password_secret env_config_maps env_secrets",
+        "instance_config_map postgres_password_secret env_config_maps env_secrets volume_mounts",
     )
 ):
     """Configuration parameters for launching Dagster Jobs on Kubernetes.
@@ -198,7 +201,7 @@ class DagsterK8sJobConfig(
         instance_config_map (str): The ``name`` of an existing Volume to mount into the pod in
             order to provide a ConfigMap for the Dagster instance. This Volume should contain a
             ``dagster.yaml`` with appropriate values for run storage, event log storage, etc.
-        postgres_password_secret (str): The name of the Kubernetes Secret where the postgres
+        postgres_password_secret (Optional[str]): The name of the Kubernetes Secret where the postgres
             password can be retrieved. Will be mounted and supplied as an environment variable to
             the Job Pod.
         env_config_maps (Optional[List[str]]): A list of custom ConfigMapEnvSource names from which to
@@ -221,6 +224,7 @@ class DagsterK8sJobConfig(
         postgres_password_secret=None,
         env_config_maps=None,
         env_secrets=None,
+        volume_mounts=None,
     ):
         return super(DagsterK8sJobConfig, cls).__new__(
             cls,
@@ -234,25 +238,24 @@ class DagsterK8sJobConfig(
             ),
             service_account_name=check.opt_str_param(service_account_name, "service_account_name"),
             instance_config_map=check.str_param(instance_config_map, "instance_config_map"),
-            postgres_password_secret=check.str_param(
+            postgres_password_secret=check.opt_str_param(
                 postgres_password_secret, "postgres_password_secret"
             ),
             env_config_maps=check.opt_list_param(env_config_maps, "env_config_maps", of_type=str),
             env_secrets=check.opt_list_param(env_secrets, "env_secrets", of_type=str),
+            volume_mounts=check.opt_list_param(volume_mounts, "volume_mounts"),
         )
 
     @classmethod
     def config_type(cls):
-        """Combined config type which includes both run launcher and pipeline run config.
-        """
+        """Combined config type which includes both run launcher and pipeline run config."""
         cfg_run_launcher = DagsterK8sJobConfig.config_type_run_launcher()
         cfg_pipeline_run = DagsterK8sJobConfig.config_type_pipeline_run()
         return merge_dicts(cfg_run_launcher, cfg_pipeline_run)
 
     @classmethod
     def config_type_run_launcher(cls):
-        """Configuration intended to be set on the Dagster instance.
-        """
+        """Configuration intended to be set on the Dagster instance."""
         return {
             "instance_config_map": Field(
                 StringSource,
@@ -263,7 +266,7 @@ class DagsterK8sJobConfig(
             ),
             "postgres_password_secret": Field(
                 StringSource,
-                is_required=True,
+                is_required=False,
                 description="The name of the Kubernetes Secret where the postgres password can be "
                 "retrieved. Will be mounted and supplied as an environment variable to the Job Pod."
                 'Secret must contain the key ``"postgresql-password"`` which will be exposed in '
@@ -281,8 +284,7 @@ class DagsterK8sJobConfig(
 
     @classmethod
     def config_type_pipeline_run(cls):
-        """Configuration intended to be set at pipeline execution time.
-        """
+        """Configuration intended to be set at pipeline execution time."""
         return {
             "job_image": Field(
                 Noneable(StringSource),
@@ -352,6 +354,9 @@ class DagsterK8sJobConfig(
     def to_dict(self):
         return self._asdict()
 
+    def with_image(self, image):
+        return self._replace(job_image=image)
+
     @staticmethod
     def from_dict(config=None):
         check.opt_dict_param(config, "config")
@@ -360,7 +365,6 @@ class DagsterK8sJobConfig(
 
 def construct_dagster_k8s_job(
     job_config,
-    command,
     args,
     job_name,
     user_defined_k8s_config=None,
@@ -373,7 +377,6 @@ def construct_dagster_k8s_job(
     Args:
         job_config (DagsterK8sJobConfig): Job configuration to use for constructing the Kubernetes
             Job object.
-        command (List[str]): Command to run.
         args (List[str]): CLI arguments to use with dagster-graphql in this Job.
         job_name (str): The name of the Job. Note that this name must be <= 63 characters in length.
         resources (Dict[str, Dict[str, str]]): The resource requirements for the container
@@ -387,7 +390,6 @@ def construct_dagster_k8s_job(
         kubernetes.client.V1Job: A Kubernetes Job object.
     """
     check.inst_param(job_config, "job_config", DagsterK8sJobConfig)
-    command = check.list_param(command, "command", of_type=str)
     check.list_param(args, "args", of_type=str)
     check.str_param(job_name, "job_name")
     user_defined_k8s_config = check.opt_inst_param(
@@ -424,6 +426,19 @@ def construct_dagster_k8s_job(
     if component:
         dagster_labels["app.kubernetes.io/component"] = component
 
+    env = [kubernetes.client.V1EnvVar(name="DAGSTER_HOME", value=job_config.dagster_home)]
+    if job_config.postgres_password_secret:
+        env.append(
+            kubernetes.client.V1EnvVar(
+                name=DAGSTER_PG_PASSWORD_ENV_VAR,
+                value_from=kubernetes.client.V1EnvVarSource(
+                    secret_key_ref=kubernetes.client.V1SecretKeySelector(
+                        name=job_config.postgres_password_secret, key=DAGSTER_PG_PASSWORD_SECRET_KEY
+                    )
+                ),
+            )
+        )
+
     additional_k8s_env_vars = []
     if env_vars:
         for key, value in env_vars.items():
@@ -432,21 +447,9 @@ def construct_dagster_k8s_job(
     job_container = kubernetes.client.V1Container(
         name=job_name,
         image=job_config.job_image,
-        command=command,
         args=args,
         image_pull_policy=job_config.image_pull_policy,
-        env=[
-            kubernetes.client.V1EnvVar(name="DAGSTER_HOME", value=job_config.dagster_home),
-            kubernetes.client.V1EnvVar(
-                name=DAGSTER_PG_PASSWORD_ENV_VAR,
-                value_from=kubernetes.client.V1EnvVarSource(
-                    secret_key_ref=kubernetes.client.V1SecretKeySelector(
-                        name=job_config.postgres_password_secret, key=DAGSTER_PG_PASSWORD_SECRET_KEY
-                    )
-                ),
-            ),
-        ]
-        + additional_k8s_env_vars,
+        env=env + additional_k8s_env_vars,
         env_from=job_config.env_from_sources,
         volume_mounts=[
             kubernetes.client.V1VolumeMount(
@@ -456,14 +459,37 @@ def construct_dagster_k8s_job(
                 ),
                 sub_path="dagster.yaml",
             )
+        ]
+        + [
+            kubernetes.client.V1VolumeMount(
+                name=mount["name"],
+                mount_path=mount["path"],
+                sub_path=mount["sub_path"],
+            )
+            for mount in job_config.volume_mounts
         ],
-        **user_defined_k8s_config.container_config
+        **user_defined_k8s_config.container_config,
     )
 
-    config_map_volume = kubernetes.client.V1Volume(
-        name="dagster-instance",
-        config_map=kubernetes.client.V1ConfigMapVolumeSource(name=job_config.instance_config_map),
-    )
+    volumes = [
+        kubernetes.client.V1Volume(
+            name="dagster-instance",
+            config_map=kubernetes.client.V1ConfigMapVolumeSource(
+                name=job_config.instance_config_map
+            ),
+        )
+    ] + [
+        kubernetes.client.V1Volume(
+            name=mount["name"],
+            config_map=kubernetes.client.V1ConfigMapVolumeSource(name=mount["configmap"]),
+        )
+        if mount.get("configmap")
+        else kubernetes.client.V1Volume(
+            name=mount["name"],
+            secret=kubernetes.client.V1SecretVolumeSource(secret_name=mount["secret"]),
+        )
+        for mount in job_config.volume_mounts
+    ]
 
     # If the user has defined custom labels, remove them from the pod_template_spec_metadata
     # key and merge them with the dagster labels
@@ -475,7 +501,7 @@ def construct_dagster_k8s_job(
         metadata=kubernetes.client.V1ObjectMeta(
             name=pod_name,
             labels=merge_dicts(dagster_labels, user_defined_pod_template_labels),
-            **user_defined_k8s_config.pod_template_spec_metadata
+            **user_defined_k8s_config.pod_template_spec_metadata,
         ),
         spec=kubernetes.client.V1PodSpec(
             image_pull_secrets=[
@@ -485,8 +511,8 @@ def construct_dagster_k8s_job(
             service_account_name=job_config.service_account_name,
             restart_policy="Never",
             containers=[job_container],
-            volumes=[config_map_volume],
-            **user_defined_k8s_config.pod_spec_config
+            volumes=volumes,
+            **user_defined_k8s_config.pod_spec_config,
         ),
     )
 
@@ -500,9 +526,9 @@ def construct_dagster_k8s_job(
             template=template,
             backoff_limit=K8S_JOB_BACKOFF_LIMIT,
             ttl_seconds_after_finished=K8S_JOB_TTL_SECONDS_AFTER_FINISHED,
-            **user_defined_k8s_config.job_spec_config
+            **user_defined_k8s_config.job_spec_config,
         ),
-        **user_defined_k8s_config.job_config
+        **user_defined_k8s_config.job_config,
     )
     return job
 
@@ -521,6 +547,6 @@ def get_k8s_job_name(input_1, input_2=None):
         input_2 = "".join(random.choice(letters) for i in range(20))
 
     # Creates 32-bit signed int, so could be negative
-    name_hash = hashlib.md5(six.ensure_binary(input_1 + input_2))
+    name_hash = hashlib.md5((input_1 + input_2).encode("utf-8"))
 
     return name_hash.hexdigest()
