@@ -105,7 +105,6 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         env_config_maps=None,
         env_secrets=None,
         k8s_client_batch_api=None,
-        k8s_client_core_api=None,
     ):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self.job_namespace = check.str_param(job_namespace, "job_namespace")
@@ -120,8 +119,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             check.opt_str_param(kubeconfig_file, "kubeconfig_file")
             kubernetes.config.load_kube_config(kubeconfig_file)
 
-        self._batch_api = k8s_client_batch_api or kubernetes.client.BatchV1Api()
-        self._core_api = k8s_client_core_api or kubernetes.client.CoreV1Api()
+        self._fixed_batch_api = k8s_client_batch_api
 
         self._job_config = None
         self._job_image = check.opt_str_param(job_image, "job_image")
@@ -142,6 +140,10 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
 
         super().__init__()
 
+    @property
+    def _batch_api(self):
+        return self._fixed_batch_api if self._fixed_batch_api else kubernetes.client.BatchV1Api()
+
     @classmethod
     def config_type(cls):
         """Include all arguments required for DagsterK8sJobConfig along with additional arguments
